#
# def_deb-fishing.R, 10 Sep 23

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)


# Data extracted from
# Identifying Error-Prone Software-An Empirical Study
# VINCENT Y. SHEN, TZE-JIE YU, STEPHEN M. THEBAUT, AND LORRI R. PAULSEN
# Fig 4, papers says 108 modules

sh85=read.csv("shen1985-fig4.csv", as.is=TRUE)

sh85$defects=sh85$defect_den*sh85$prog_len

table(round(sh85$defects*4, digits=2))

# The original plot had no axis values
plot(sh85$prog_len, sh85$defects, log="xy", col=pal_col[2], cex=1.5,
	xaxt="n", yaxt="n",
	xlab="Program length", ylab="Defects")

x_range=1:100
sh85_mod=glm(log(defects) ~ log(prog_len), data=sh85)
summary(sh85_mod)
pred=predict(sh85_mod, newdata=data.frame(prog_len=x_range))

lines(x_range, exp(pred), col=pal_col[1])

dev.copy(dev=png, file="shen85_def-den.png")
dev.off()


# Data extracted from
# Error Density and Size in Ada Software
# Carol Withrow
# Fig 2, paper says 137 modules 
wi90=read.csv("withrow1990.csv",as.is=TRUE)

wi90$LOC=10^(wi90$log_loc)
wi90$errors=trunc(10^(wi90$log_err_1kloc)*LOC/1000+0.2)

plot(wi90$LOC, wi90$errors, log="xy", col=pal_col[2], cex=1.5,
	xlab="LOC in package", ylab="Errors")

x_range=1:1e4
wi90_mod=glm(log(errors) ~ log(LOC), data=wi90, subset=(LOC > 125))
summary(wi90_mod)
pred=predict(wi90_mod, newdata=data.frame(LOC=x_range))

lines(x_range, exp(pred), col=pal_col[1])


dev.copy(dev=png, file="with90_def-den.png")
dev.off()


# Data extracted from
# An Empirical Investigation of Software Fault Distribution
# Karl-Heinrich Moller and Daniel J. Paulish
# Fig 5

mo93=read.csv("moller1993.csv",as.is=TRUE)


